@@ -54,9 +54,9 @@ from ormar.fields import (
     UUID,
     UniqueColumns,
 )  # noqa: I100
-from ormar.models import Model
+from ormar.models import ExcludableItems, Model
 from ormar.models.metaclass import ModelMeta
-from ormar.queryset import QuerySet
+from ormar.queryset import OrderAction, QuerySet
 from ormar.relations import RelationType
 from ormar.signals import Signal
 
@@ -68,7 +68,7 @@ class UndefinedType:  # pragma no cover
 
 Undefined = UndefinedType()
 
-__version__ = "0.9.5"
+__version__ = "0.9.6"
 __all__ = [
     "Integer",
     "BigInteger",
@@ -106,4 +106,6 @@ __all__ = [
     "BaseField",
     "ManyToManyField",
     "ForeignKeyField",
+    "OrderAction",
+    "ExcludableItems",
 ]
