@@ -1,10 +1,19 @@
 """
 Contains QuerySet and different Query classes to allow for constructing of sql queries.
 """
+from ormar.queryset.actions import FilterAction, OrderAction
 from ormar.queryset.filter_query import FilterQuery
 from ormar.queryset.limit_query import LimitQuery
 from ormar.queryset.offset_query import OffsetQuery
 from ormar.queryset.order_query import OrderQuery
 from ormar.queryset.queryset import QuerySet
 
-__all__ = ["QuerySet", "FilterQuery", "LimitQuery", "OffsetQuery", "OrderQuery"]
+__all__ = [
+    "QuerySet",
+    "FilterQuery",
+    "LimitQuery",
+    "OffsetQuery",
+    "OrderQuery",
+    "FilterAction",
+    "OrderAction",
+]
