@@ -1,4 +1,5 @@
-from typing import (
+from _weakref import CallableProxyType
+from typing import (  # noqa: I100, I201
     Any,
     Dict,
     List,
@@ -7,12 +8,12 @@ from typing import (
     Sequence,
     Set,
     TYPE_CHECKING,
-    TypeVar,
     Union,
+    cast,
 )
 
 import ormar
-from ormar.exceptions import ModelPersistenceError
+from ormar.exceptions import ModelPersistenceError, QueryDefinitionError
 
 if TYPE_CHECKING:  # pragma no cover
     from ormar.relations import Relation
@@ -20,10 +21,8 @@ if TYPE_CHECKING:  # pragma no cover
     from ormar.queryset import QuerySet
     from ormar import RelationType
 
-    T = TypeVar("T", bound=Model)
 
-
-class QuerysetProxy(ormar.QuerySetProtocol):
+class QuerysetProxy:
     """
     Exposes QuerySet methods on relations, but also handles creating and removing
     of through Models for m2m relations.
@@ -38,12 +37,17 @@ class QuerysetProxy(ormar.QuerySetProtocol):
         self.relation: Relation = relation
         self._queryset: Optional["QuerySet"] = qryset
         self.type_: "RelationType" = type_
-        self._owner: "Model" = self.relation.manager.owner
+        self._owner: Union[CallableProxyType, "Model"] = self.relation.manager.owner
         self.related_field_name = self._owner.Meta.model_fields[
             self.relation.field_name
         ].get_related_name()
         self.related_field = self.relation.to.Meta.model_fields[self.related_field_name]
         self.owner_pk_value = self._owner.pk
+        self.through_model_name = (
+            self.related_field.through.get_name()
+            if self.type_ == ormar.RelationType.MULTIPLE
+            else ""
+        )
 
     @property
     def queryset(self) -> "QuerySet":
@@ -65,7 +69,7 @@ class QuerysetProxy(ormar.QuerySetProtocol):
         """
         self._queryset = value
 
-    def _assign_child_to_parent(self, child: Optional["T"]) -> None:
+    def _assign_child_to_parent(self, child: Optional["Model"]) -> None:
         """
         Registers child in parents RelationManager.
 
@@ -77,7 +81,9 @@ class QuerysetProxy(ormar.QuerySetProtocol):
             rel_name = self.relation.field_name
             setattr(owner, rel_name, child)
 
-    def _register_related(self, child: Union["T", Sequence[Optional["T"]]]) -> None:
+    def _register_related(
+        self, child: Union["Model", Sequence[Optional["Model"]]]
+    ) -> None:
         """
         Registers child/ children in parents RelationManager.
 
@@ -89,6 +95,7 @@ class QuerysetProxy(ormar.QuerySetProtocol):
                 self._assign_child_to_parent(subchild)
         else:
             assert isinstance(child, ormar.Model)
+            child = cast("Model", child)
             self._assign_child_to_parent(child)
 
     def _clean_items_on_load(self) -> None:
@@ -99,17 +106,20 @@ class QuerysetProxy(ormar.QuerySetProtocol):
             for item in self.relation.related_models[:]:
                 self.relation.remove(item)
 
-    async def create_through_instance(self, child: "T") -> None:
+    async def create_through_instance(self, child: "Model", **kwargs: Any) -> None:
         """
         Crete a through model instance in the database for m2m relations.
 
+        :param kwargs: dict of additional keyword arguments for through instance
+        :type kwargs: Any
         :param child: child model instance
         :type child: Model
         """
         model_cls = self.relation.through
         owner_column = self.related_field.default_target_field_name()  # type: ignore
         child_column = self.related_field.default_source_field_name()  # type: ignore
-        kwargs = {owner_column: self._owner.pk, child_column: child.pk}
+        rel_kwargs = {owner_column: self._owner.pk, child_column: child.pk}
+        final_kwargs = {**rel_kwargs, **kwargs}
         if child.pk is None:
             raise ModelPersistenceError(
                 f"You cannot save {child.get_name()} "
@@ -117,18 +127,34 @@ class QuerysetProxy(ormar.QuerySetProtocol):
                 f"Save the child model first."
             )
         expr = model_cls.Meta.table.insert()
-        expr = expr.values(**kwargs)
+        expr = expr.values(**final_kwargs)
         # print("\n", expr.compile(compile_kwargs={"literal_binds": True}))
         await model_cls.Meta.database.execute(expr)
 
-    async def delete_through_instance(self, child: "T") -> None:
+    async def update_through_instance(self, child: "Model", **kwargs: Any) -> None:
+        """
+        Updates a through model instance in the database for m2m relations.
+
+        :param kwargs: dict of additional keyword arguments for through instance
+        :type kwargs: Any
+        :param child: child model instance
+        :type child: Model
+        """
+        model_cls = self.relation.through
+        owner_column = self.related_field.default_target_field_name()  # type: ignore
+        child_column = self.related_field.default_source_field_name()  # type: ignore
+        rel_kwargs = {owner_column: self._owner.pk, child_column: child.pk}
+        through_model = await model_cls.objects.get(**rel_kwargs)
+        await through_model.update(**kwargs)
+
+    async def delete_through_instance(self, child: "Model") -> None:
         """
         Removes through model instance from the database for m2m relations.
 
         :param child: child model instance
         :type child: Model
         """
-        queryset = ormar.QuerySet(model_cls=self.relation.through)
+        queryset = ormar.QuerySet(model_cls=self.relation.through)  # type: ignore
         owner_column = self.related_field.default_target_field_name()  # type: ignore
         child_column = self.related_field.default_source_field_name()  # type: ignore
         kwargs = {owner_column: self._owner, child_column: child}
@@ -176,10 +202,10 @@ class QuerysetProxy(ormar.QuerySetProtocol):
         :rtype: int
         """
         if self.type_ == ormar.RelationType.MULTIPLE:
-            queryset = ormar.QuerySet(model_cls=self.relation.through)
+            queryset = ormar.QuerySet(model_cls=self.relation.through)  # type: ignore
             owner_column = self._owner.get_name()
         else:
-            queryset = ormar.QuerySet(model_cls=self.relation.to)
+            queryset = ormar.QuerySet(model_cls=self.relation.to)  # type: ignore
             owner_column = self.related_field.name
         kwargs = {owner_column: self._owner}
         self._clean_items_on_load()
@@ -270,14 +296,47 @@ class QuerysetProxy(ormar.QuerySetProtocol):
         :return: created model
         :rtype: Model
         """
+        through_kwargs = kwargs.pop(self.through_model_name, {})
         if self.type_ == ormar.RelationType.REVERSE:
             kwargs[self.related_field.name] = self._owner
         created = await self.queryset.create(**kwargs)
         self._register_related(created)
         if self.type_ == ormar.RelationType.MULTIPLE:
-            await self.create_through_instance(created)
+            await self.create_through_instance(created, **through_kwargs)
         return created
 
+    async def update(self, each: bool = False, **kwargs: Any) -> int:
+        """
+        Updates the model table after applying the filters from kwargs.
+
+        You have to either pass a filter to narrow down a query or explicitly pass
+        each=True flag to affect whole table.
+
+        :param each: flag if whole table should be affected if no filter is passed
+        :type each: bool
+        :param kwargs: fields names and proper value types
+        :type kwargs: Any
+        :return: number of updated rows
+        :rtype: int
+        """
+        # queryset proxy always have one filter for pk of parent model
+        if not each and len(self.queryset.filter_clauses) == 1:
+            raise QueryDefinitionError(
+                "You cannot update without filtering the queryset first. "
+                "If you want to update all rows use update(each=True, **kwargs)"
+            )
+
+        through_kwargs = kwargs.pop(self.through_model_name, {})
+        children = await self.queryset.all()
+        for child in children:
+            await child.update(**kwargs)  # type: ignore
+            if self.type_ == ormar.RelationType.MULTIPLE and through_kwargs:
+                await self.update_through_instance(
+                    child=child,  # type: ignore
+                    **through_kwargs,
+                )
+        return len(children)
+
     async def get_or_create(self, **kwargs: Any) -> "Model":
         """
         Combination of create and get methods.
